@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2018 GuardSquare NV
+ * Copyright (c) 2002-2019 Guardsquare NV
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -73,6 +73,9 @@ public class ConfigurationTask extends Task
         configuration.assumeNoExternalReturnValues = extendClassSpecifications(configuration.assumeNoExternalReturnValues,
                                                                                this.configuration.assumeNoExternalReturnValues);
 
+        configuration.assumeValues                 = extendClassSpecifications(configuration.assumeValues,
+                                                                               this.configuration.assumeValues);
+
         configuration.keepPackageNames             = extendList(configuration.keepPackageNames,
                                                                 this.configuration.keepPackageNames);
 
@@ -234,6 +237,13 @@ public class ConfigurationTask extends Task
     }
 
 
+    public void addConfiguredAssumevalues(ClassSpecificationElement classSpecificationElement)
+    {
+        configuration.assumeValues = extendClassSpecifications(configuration.assumeValues,
+                                                               classSpecificationElement);
+    }
+
+
     public void addConfiguredOptimizations(FilterElement filterElement)
     {
         addConfiguredOptimization(filterElement);
