@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2018 GuardSquare NV
+ * Copyright (c) 2002-2019 Guardsquare NV
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -1498,8 +1498,8 @@ implements   AttributeVisitor,
      * This pseudo-instruction represents a label that marks an instruction
      * offset, for use in the context of the code attribute editor only.
      */
-    private static class Label
-    extends              Instruction
+    public static class Label
+    extends             Instruction
     {
         protected final int identifier;
 
