@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2018 GuardSquare NV
+ * Copyright (c) 2002-2019 Guardsquare NV
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -142,6 +142,17 @@ implements   ClassVisitor,
     }
 
 
+    public void visitDynamicConstant(Clazz clazz, DynamicConstant dynamicConstant)
+    {
+        markAsUsed(dynamicConstant);
+
+        markConstant(clazz, dynamicConstant.u2nameAndTypeIndex);
+
+        // Mark the bootstrap methods attribute.
+        clazz.attributesAccept(this);
+    }
+
+
     public void visitInvokeDynamicConstant(Clazz clazz, InvokeDynamicConstant invokeDynamicConstant)
     {
         markAsUsed(invokeDynamicConstant);
@@ -253,34 +264,42 @@ implements   ClassVisitor,
 
     public void visitEnclosingMethodAttribute(Clazz clazz, EnclosingMethodAttribute enclosingMethodAttribute)
     {
-        markConstant(clazz, enclosingMethodAttribute.u2attributeNameIndex);
-        markConstant(clazz, enclosingMethodAttribute.u2classIndex);
+        markConstant(        clazz, enclosingMethodAttribute.u2attributeNameIndex);
+        markConstant(        clazz, enclosingMethodAttribute.u2classIndex);
+        markOptionalConstant(clazz, enclosingMethodAttribute.u2nameAndTypeIndex);
+    }
 
-        if (enclosingMethodAttribute.u2nameAndTypeIndex != 0)
-        {
-            markConstant(clazz, enclosingMethodAttribute.u2nameAndTypeIndex);
-        }
+
+    public void visitNestHostAttribute(Clazz clazz, NestHostAttribute nestHostAttribute)
+    {
+        markConstant(clazz, nestHostAttribute.u2attributeNameIndex);
+        markConstant(clazz, nestHostAttribute.u2hostClassIndex);
+    }
+
+
+    public void visitNestMembersAttribute(Clazz clazz, NestMembersAttribute nestMembersAttribute)
+    {
+        markConstant(clazz, nestMembersAttribute.u2attributeNameIndex);
+
+        // Mark the nest member class constants.
+        nestMembersAttribute.memberClassConstantsAccept(clazz, this);
     }
 
 
     public void visitModuleAttribute(Clazz clazz, ModuleAttribute moduleAttribute)
     {
-        markConstant(clazz, moduleAttribute.u2attributeNameIndex);
-        markConstant(clazz, moduleAttribute.u2moduleNameIndex);
+        markConstant(        clazz, moduleAttribute.u2attributeNameIndex);
+        markConstant(        clazz, moduleAttribute.u2moduleNameIndex);
+        markOptionalConstant(clazz, moduleAttribute.u2moduleVersionIndex);
 
-        if (moduleAttribute.u2moduleVersionIndex != 0)
-        {
-            markConstant(clazz, moduleAttribute.u2moduleVersionIndex);
-        }
+        // Mark the constant pool entries referenced by the contained info.
         moduleAttribute.requiresAccept(clazz, this);
         moduleAttribute.exportsAccept(clazz, this);
         moduleAttribute.opensAccept(clazz, this);
 
-        for (int index = 0; index < moduleAttribute.u2usesCount; index++)
-        {
-            markConstant(clazz, moduleAttribute.u2uses[index]);
-        }
+        markConstants(clazz, moduleAttribute.u2uses, moduleAttribute.u2usesCount);
 
+        // Mark the constant pool entries referenced by the provides info.
         moduleAttribute.providesAccept(clazz, this);
     }
 
@@ -296,10 +315,8 @@ implements   ClassVisitor,
     {
         markConstant(clazz, modulePackagesAttribute.u2attributeNameIndex);
 
-        for (int index = 0; index < modulePackagesAttribute.u2packagesCount; index++)
-        {
-            markConstant(clazz, modulePackagesAttribute.u2packages[index]);
-        }
+        // Mark the constant pool entries referenced by the packages info.
+        modulePackagesAttribute.packagesAccept(clazz, this);
     }
 
 
@@ -425,6 +442,7 @@ implements   ClassVisitor,
 
     public void visitInnerClassesInfo(Clazz clazz, InnerClassesInfo innerClassesInfo)
     {
+        // Mark the constant pool entries referenced by the contained info.
         innerClassesInfo.innerClassConstantAccept(clazz, this);
         innerClassesInfo.outerClassConstantAccept(clazz, this);
         innerClassesInfo.innerNameConstantAccept(clazz, this);
@@ -435,10 +453,7 @@ implements   ClassVisitor,
 
     public void visitExceptionInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, ExceptionInfo exceptionInfo)
     {
-        if (exceptionInfo.u2catchType != 0)
-        {
-            markConstant(clazz, exceptionInfo.u2catchType);
-        }
+        markOptionalConstant(clazz, exceptionInfo.u2catchType);
     }
 
 
@@ -484,10 +499,7 @@ implements   ClassVisitor,
 
     public void visitParameterInfo(Clazz clazz, Method method, int parameterIndex, ParameterInfo parameterInfo)
     {
-        if (parameterInfo.u2nameIndex != 0)
-        {
-            markConstant(clazz, parameterInfo.u2nameIndex);
-        }
+        markOptionalConstant(clazz, parameterInfo.u2nameIndex);
     }
 
 
@@ -513,8 +525,8 @@ implements   ClassVisitor,
 
     public void visitRequiresInfo(Clazz clazz, RequiresInfo requiresInfo)
     {
-        markConstant(clazz, requiresInfo.u2requiresIndex);
-        markConstant(clazz, requiresInfo.u2requiresVersionIndex);
+        markConstant(        clazz, requiresInfo.u2requiresIndex);
+        markOptionalConstant(clazz, requiresInfo.u2requiresVersionIndex);
     }
 
 
@@ -522,12 +534,8 @@ implements   ClassVisitor,
 
     public void visitExportsInfo(Clazz clazz, ExportsInfo exportsInfo)
     {
-        markConstant(clazz, exportsInfo.u2exportsIndex);
-
-        for (int index = 0; index < exportsInfo.u2exportsToCount; index++)
-        {
-            markConstant(clazz, exportsInfo.u2exportsToIndex[index]);
-        }
+        markConstant( clazz,  exportsInfo.u2exportsIndex);
+        markConstants(clazz, exportsInfo.u2exportsToIndex, exportsInfo.u2exportsToCount);
     }
 
 
@@ -535,12 +543,8 @@ implements   ClassVisitor,
 
     public void visitOpensInfo(Clazz clazz, OpensInfo opensInfo)
     {
-        markConstant(clazz, opensInfo.u2opensIndex);
-
-        for (int index = 0; index < opensInfo.u2opensToCount; index++)
-        {
-            markConstant(clazz, opensInfo.u2opensToIndex[index]);
-        }
+        markConstant( clazz, opensInfo.u2opensIndex);
+        markConstants(clazz, opensInfo.u2opensToIndex, opensInfo.u2opensToCount);
     }
 
 
@@ -548,12 +552,8 @@ implements   ClassVisitor,
 
     public void visitProvidesInfo(Clazz clazz, ProvidesInfo providesInfo)
     {
-        markConstant(clazz, providesInfo.u2providesIndex);
-
-        for (int index = 0; index < providesInfo.u2providesWithCount; index++)
-        {
-            markConstant(clazz, providesInfo.u2providesWithIndex[index]);
-        }
+        markConstant( clazz, providesInfo.u2providesIndex);
+        markConstants(clazz, providesInfo.u2providesWithIndex, providesInfo.u2providesWithCount);
     }
 
 
@@ -572,44 +572,29 @@ implements   ClassVisitor,
 
     public void visitConstantElementValue(Clazz clazz, Annotation annotation, ConstantElementValue constantElementValue)
     {
-        if (constantElementValue.u2elementNameIndex != 0)
-        {
-            markConstant(clazz, constantElementValue.u2elementNameIndex);
-        }
-
-        markConstant(clazz, constantElementValue.u2constantValueIndex);
+        markOptionalConstant(clazz, constantElementValue.u2elementNameIndex);
+        markConstant(        clazz, constantElementValue.u2constantValueIndex);
     }
 
 
     public void visitEnumConstantElementValue(Clazz clazz, Annotation annotation, EnumConstantElementValue enumConstantElementValue)
     {
-        if (enumConstantElementValue.u2elementNameIndex != 0)
-        {
-            markConstant(clazz, enumConstantElementValue.u2elementNameIndex);
-        }
-
-        markConstant(clazz, enumConstantElementValue.u2typeNameIndex);
-        markConstant(clazz, enumConstantElementValue.u2constantNameIndex);
+        markOptionalConstant(clazz, enumConstantElementValue.u2elementNameIndex);
+        markConstant(        clazz, enumConstantElementValue.u2typeNameIndex);
+        markConstant(        clazz, enumConstantElementValue.u2constantNameIndex);
     }
 
 
     public void visitClassElementValue(Clazz clazz, Annotation annotation, ClassElementValue classElementValue)
     {
-        if (classElementValue.u2elementNameIndex != 0)
-        {
-            markConstant(clazz, classElementValue.u2elementNameIndex);
-        }
-
-        markConstant(clazz, classElementValue.u2classInfoIndex);
+        markOptionalConstant(clazz, classElementValue.u2elementNameIndex);
+        markConstant(        clazz, classElementValue.u2classInfoIndex);
     }
 
 
     public void visitAnnotationElementValue(Clazz clazz, Annotation annotation, AnnotationElementValue annotationElementValue)
     {
-        if (annotationElementValue.u2elementNameIndex != 0)
-        {
-            markConstant(clazz, annotationElementValue.u2elementNameIndex);
-        }
+        markOptionalConstant(clazz, annotationElementValue.u2elementNameIndex);
 
         // Mark the constant pool entries referenced by the annotation.
         annotationElementValue.annotationAccept(clazz, this);
@@ -618,10 +603,7 @@ implements   ClassVisitor,
 
     public void visitArrayElementValue(Clazz clazz, Annotation annotation, ArrayElementValue arrayElementValue)
     {
-        if (arrayElementValue.u2elementNameIndex != 0)
-        {
-            markConstant(clazz, arrayElementValue.u2elementNameIndex);
-        }
+        markOptionalConstant(clazz, arrayElementValue.u2elementNameIndex);
 
         // Mark the constant pool entries referenced by the element values.
         arrayElementValue.elementValuesAccept(clazz, annotation, this);
@@ -642,12 +624,40 @@ implements   ClassVisitor,
     // Small utility methods.
 
     /**
-     * Marks the given constant pool entry of the given class. This includes
-     * visiting any referenced objects.
+     * Marks the specified constant pool entries of the given class.
+     * This includes visiting any referenced objects.
      */
-    private void markConstant(Clazz clazz, int index)
+    private void markConstants(Clazz clazz,
+                               int[] constantIndices,
+                               int   constantIndicesCount)
     {
-        clazz.constantPoolEntryAccept(index, this);
+        for (int index = 0; index < constantIndicesCount; index++)
+        {
+            markConstant(clazz, constantIndices[index]);
+        }
+    }
+
+
+    /**
+     * Marks the specified constant pool entry of the given class, if the index
+     * is not 0. This includes visiting any referenced objects.
+     */
+    private void markOptionalConstant(Clazz clazz, int constantIndex)
+    {
+        if (constantIndex != 0)
+        {
+            markConstant(clazz, constantIndex);
+        }
+    }
+
+
+    /**
+     * Marks the specified constant pool entry of the given class.
+     * This includes visiting any referenced objects.
+     */
+    private void markConstant(Clazz clazz, int constantIndex)
+    {
+        clazz.constantPoolEntryAccept(constantIndex, this);
     }
 
 
